@@ -974,6 +974,7 @@ sync_file(const gchar *pathname, GError **error)
 	if (fdatasync(fd) == -1)
 	{
 		set_error_from_errno(error);
+		close(fd);
 		return -1;
 	}
 	if (close(fd) == -1)
@@ -1338,8 +1339,8 @@ static char *parse_string_mask(const char *mask, int len, GError **error)
 
 	return parsed_mask;
 err:
-	g_return_val_if_fail(error == NULL || *error != NULL, NULL);
 	g_free(parsed_mask);
+	g_return_val_if_fail(error == NULL || *error != NULL, NULL);
 	return NULL;
 }
 
@@ -2239,6 +2240,8 @@ write_data (FILE *cache, const gchar *n, gint len)
 
   i = fwrite (s, l, 1, cache);
 
+  g_free(s);
+
   return i == 1;
   
 }
